# README for Multiphase Python Repository by HZDR

[[_TOC_]]

The python package provides several routines and scripts required to operate
the code and cases repositories containing additional code and set-ups for the
open-source software released by the OpenFOAM Foundation. This includes among
others utilities for pre- and post-processing of simulation cases, utilities
to launch virtual environments containing the source code, and utilities to
operate the continuous integration and continuous development environment in a
self-hosted Gitlab instance.

## Utilities

The module contains several stand-alone python scripts. For more information on
each script use the `--help` option.

| Name       | Description                                                                         |
|------------|-------------------------------------------------------------------------------------|
| mpyconv    | Converts data files between readable and writable formats defined in multiphasepy. |
| mpycopy    | Renders a ready-to-run case setup from a template using copier.                     |
| mpydocker  | Launch a customizable docker container for a given image from the registry.         |
| mpyhooks   | Provides various hooks to be executed as git-hook e.g., via pre-commit.             |
| mpyfuzzy   | Evaluate goodness for a simulation case by using fuzzy logic.                       |
| mpypost    | Run some math as post-processing on data files.                                     |
| mpypublish | Publish a software publication in Rossendorf Data Repository (Rodare).              |
| mpyrpcmp   | Compares two repositories and print result with respect to the left repository.     |
| mpyshrun   | Wraps a given command similar to runApplication from RunFunctions.                  |

## Installation

Note that a Python version >= 3.11 is required. Install prerequisites via

```bash
sudo apt-get update
sudo apt-get install python3-full python3-dev git
```

For installing the multiphasepy package create a Python virtual environment

```bash
python3 -m venv <PATH/TO/VENV>
```

If you wish to make Python packages available in the environment that are
installed system-wide, create the environment with the additional option
`--system-site-packages`

```bash
python3 -m venv --system-site-packages <PATH/TO/VENV>
```

Environments are activated by

```bash
. <PATH/TO/VENV>/bin/activate
```

and deactivated by

```bash
deactivate
```

The multiphasepy package is installed via pip[^1] from the Gitlab PyPi. For a
simple and convenient usage add the following to `$HOME/.config/pip/pip.conf`

```bash
[global]
extra-index-url = https://codebase.helmholtz.cloud/api/v4/projects/8786/packages/pypi/simple
```

With the environment activated, install the package with

```bash
pip install multiphasepy
```

or upgrade with

```bash
pip install --upgrade multiphasepy
```

If required pip can install extra (recommended) dependencies for python

```bash
pip install multiphasepy[<extra>]
```

Note, you have to replace `<extra>` with an extra from the table below

| Extra | Description                                                                    |
|-------|--------------------------------------------------------------------------------|
| all   | All extra packages                                                             |
| rec   | Packages recommended for daily usage, e.g., to run a workflow with Snakemake   |
| dev   | Packages required for development of the package, e.g., pytest and ruff linter |

## Links and References

[^1]: <https://pypi.org/project/pip/>
